% BUILD_JTREE --- Build a hierarchical/wavelet tree of 2-by-2 Jacobi rotations
%
% Usage:
% [Zpos, T, PCidx, all_nodes, theta, PC_ratio, Z, all_d, Cnew, ccnew] = Build_JTree(C,cc,maxlevel)
 

% Input:
%   C(dim,dim)      input covariance matrix
%   cc(dim,dim)     correlation coeffs
%   maxlevel        default: dim-1 (levels L=0,1,2,...,maxlevel)
%
% Output:
%   Zpos(dim-1,2)       (UINT16) positions of the two clusters merged;
%                       number between 1 and d (the length of the comp vector)
%   T{dim-1,2}          Element T{k} in the cell array contains a 2-by-2 rot matrix 
%   PCidx(dim-1,2)      (UINT8) ID of prinicipal components; [1 2] or [2 1]
%   all_nodes(dim-1,dim)(UINT16)   node labels
%   theta(dim-1,1)      rotation angles that decorrelates inputs X(:,Z(k,1)) and X(:,Z(k,2))
%   PC_ratio(dim-1,1)   ratio C_qq/C_pp, where p is the index of the 1st
%                       principal component and q is the index of the 2nd principal comp
%   Z(dim-1,3)          hierarchical cluster tree; columns 1 and 2 contain the
%                       indices of clusters that are merged; 3rd column
%                       contains a distance measure between 0 and 1
%   all_d(dim-1,dim)    (UINT16)  labels of the d-components in the comp
%                       vector (=0 if s-component, otherwise 1,...,d-1)
%   Cnew(dim,dim)       updated covariance matrix 
%   ccnew(dim,dim)      correlation coefficients
%
%
%
% Description:
%   Builds a hierarchical binary tree using the covariance structure of the inputs. 
%    Decorrelates groups of two with PCA (Givens/Jacobi rotations).
%
% Calls
%   newJacobi.m
%
% By Ann B. Lee, Yale University, Jan 2003. 
% Last changed, ABL (Carnegie Mellon): 1/5/05, 4/15/08

function [Zpos, T, PCidx, all_nodes, theta, PC_ratio, Z, all_d, Cnew, ccnew] = Build_JTree(C,cc,maxlevel)


[dim,dim2] = size(C);   % C(dim,dim)
if dim>65535
    disp('ERROR: Too large matrix');
end
if (dim ~= dim2)
    fprintf(1,'Wrong dimensions of input matrix\n');
end

if nargin<3
    J = dim-1;   % max level (L=0,1,...,J); fine-to-coarse
else
    J=maxlevel;
end

Z = zeros(J,3);    % columns 1 and 2 contain the indices of the nodes that are merged
T = cell(J,1);  % 2-by-2 rotation matrices for levels L=0,1,..,J-1
theta = zeros(J,1); % rotation angles
PCidx=uint8(zeros(J,2));  % order of principal components, [1 2] or [2 1]

% INITIALIZATION:
L=1; % number of nodes = dim-lev
maskno=[];  % NOTE: only transform scaling functions! (mask out wavelet functions)
nodes=uint16(1:dim);  % node indices
dlabels=uint16(zeros(dim,1)); %-1*ones(dim,1);
PC_ratio=zeros(dim-1,1);
Zpos=uint16(zeros(J,2));
all_d=uint16(zeros(J,dim));   all_nodes=uint16(zeros(J,dim));

for lev=1:J      % loop through scale levels

    [Cnew, ccnew, R, th, compno, maxcc,idx] = newJacobi(C,cc,maskno);
    
    %--------------------------------
    % STORE OUTPUT. GO TO NEXT LEVEL
    dist = (1-maxcc)/2;  % distance between 0 and 1
    Z(lev,:) = [double(nodes(compno)), dist];  %nodes from preceding level that are merged
    T{lev} = R;   % 2-by-2 transformation matrix; Jacobi rotation
    theta(lev)=th;  % rotation angle
    PCidx(lev,:)=uint8(idx);  % [1 2] or [2 1]

    % note: C(dim,dim) => keep track of node numbers
    C=Cnew; cc=ccnew;
   
    % Various stored variables (Added, ABL 2/15/03)
    pind=compno(idx); % new node numbers for sum and diff components
    p1=pind(1); p2=pind(2); % pos of 1st and 2nd principal comps, resp
    nodes(pind)=uint16([dim+lev, 0]); % new node numbers for sum and diff comps
    dlabels(p2)=uint16(lev);  %labels of d-components in vector
    maskno=[maskno,p2];
    PC_ratio(lev)=C(p2,p2)/C(p1,p1);
    Zpos(lev,:) = uint16(compno); % their positions in the component vector
    all_d(lev,:)=dlabels'; all_nodes(lev,:)=nodes; % matrices of size JXdim
end