% JTree_NewBasis --- Calculate the basis for the Jacobi tree algorithm (one
% or more iterations)
%
% Usage:
%   [basis, sums, difs] = JTree_NewBasis(Zpos, T, PCidx, maxlev, all_nodes)
%
% Input:
%   Zpos(J,3)  (UINT16) hierarchical cluster information; output from
%           "Build_JTree"; columns 1 and 2 contain the
%           positions of the components that are merged
%   T{J,1}  Element T{k} in the cell array contains a 2-by-2 matrix 
%           that decorrelates inputs X(:,Z(k,1)) and X(:,Z(k,2));
%           output from LKLB_Decorrelate.
%   PCidx(J,2)  (UINT8) id of principal components (1st or 2nd)
%   maxlev  default:dim-1
%   all_nodes(dim-1,dim)     (UINT16) node labels
%
%
% Output:
%   basis(m,m)       Basis functions entered ROW-WISE --- coarse-to-fine 
%                     where m is the dimension of the data
%   sums(J,m)      Part of basis funcs of subspaces V1,...,V_{m-1} 
%                    ---- entered rowwise (fine-to-coarse)
%   difs(J,m)      Basis funcs of subspaces W1,...,W_{m-1}        
%                    ---- entered rowwise (fine-to-coarse)
%
% Note:
%   Original high-res data x \in V0 = V1 + W1 = Vk + Wk + ... + W1
%   where V_k = V_{k+1} + W_{k+1}
%   basis=[sums(m-1,:); difs(m-1,:); ....; difs(1,:)] 
%
% By Ann B. Lee, Yale University, Jan 2003. 
% Last changed, ABL (Carnegie Mellon University): 9/6/04, 4/15/08

function [basis, sums, difs] = JTree_Basis(Zpos,T,PCidx,maxlev,all_nodes)

if size(Zpos,1)~=size(T,1), disp('Error in the dimensions of the inputs'); end
J=size(Zpos,1); % number of input variables
if nargin<4; 
    m=J+1; % dimension of the data
else
    m=size(all_nodes,2);  % dimension of the data
    nodes = all_nodes(maxlev,:); nodes=nodes(find(nodes>0)); clear all_nodes;
end
tmpfilts=[diag(ones(m,1))];   % each row is a component/band
ind=cell(J,1);
if nargin<4
    maxlev = m-1;
end
sums=zeros(maxlev,m); difs=zeros(maxlev,m);


for lev=1:maxlev  % loop through the levels
    s=tmpfilts(Zpos(lev,1:2),:);        % s(2,m); components to be merged
    R=T{lev};                      % R(2,2); rotation matrix  
    y=R'*s; % updated: April 07
    tmpfilts(Zpos(lev,1:2),:)=y; 
    y=y(PCidx(lev,:),:); % y(1,:)=PC1, y(2,:)=PC2; sorted!
    sums(lev,:)=y(1,:);
    difs(lev,:)=y(2,:);
end
if nargin<4
    basis=[sums(J,:); flipud(difs(J-m+2:J,:))]; % coarse-to-fine *rowwise*
else
    tmp = [diag(ones(m,1)); sums];
    basis = [tmp(nodes,:); flipud(difs)]; % modified 3/18/06
end