% newJacobi --- Decorrelate group of two components with modified Jacobi's method
%            
% Usage:
%   [Cnew, ccnew, R, theta, compno, maxcc, idx] = Jacobi(C,cc,maskno)
%
% Input:
%   C(dim,dim)      input covariance matrix
%   cc(dim,dim)     correlation coeffs
%   maskno          components which are already merged
%
% Output:
%   Cnew(dim,dim)   covariance of rotated data
%   ccnew(dim,dim)  correlation coefficients for rotated data
%   R(2,2)          part of Jacobi rotation; 
%                   x <- R'x 
%                   C <- R'CR
%   theta           rotation angle in radians (clock-wise)

%   compno(1,2)     indices of components that are rotated (pick pair with
%                   highest correlation)
%   maxcc           correlation coefficient for pair
%   idx             [1 2] or [2 1] depending on order of prinicipal comps
%
% Description:
%   Picks the two components with highest correlation coefficient.
%   Decorrelate the two components with PCA
%
% By Ann B. Lee, Yale University, Jan 2003. 
% Last changed, ABL (Carnegie Mellon Univ): 9/24/03, 4/5/07, 4/15/08 
 
function [Cnew, ccnew, R, theta, compno, maxcc, idx] = newJacobi(C,cc,maskno)

   %------------------------------------------------------------
   % SELECT GROUP OF TWO COMPONENTS THAT ARE HIGHLY CORRELATED
   %------------------------------------------------------------
   if nargin<3
      maskno=[];
      if nargin<2; d=diag(C); cc=C./sqrt(d*d'); end; 
            %correlation coefficients cc(dim,dim)
   end
   tri=triu(cc,1); 
   tri(tri==0)=-1; %extract upper triangle of correlation matrix
   tri(maskno,:)=-1;  tri(:,maskno)=-1; %mask out earlier merged nodes
   [maxcc,no1] = max(tri); [maxcc,no2]=max(maxcc); no1=no1(no2); %highly correlated pair:(no1,no2)
   compno = [no1,no2]; 
   %pair=data(:,compno);   % pair(N,2)
   
   %------------------------------------
   % DECORRELATE COMPONENTS IN PAIR
   %------------------------------------
   Cred = C(compno,compno); % 2-by-2 covariance matrix
   
   if Cred(1,2)==0  % already diagonal
       Cnew=C; ccnew=cc; 
       R=[1 0; 0 1]; theta=0; idx=[1 2];
   else
       % Original Jacobi formalism
       C11=Cred(1,1); C22=Cred(2,2); C12=Cred(1,2);       
       theta = 1/2 * atan(2*C12/(C11-C22));
       
       cs=cos(theta); sn=sin(theta); 
       R = [cs -sn; sn cs];
       
       %-----------------------------------------------------
       % NEW COVARIANCE MATRIX AND CORRELATION COEFFICIENTS
       % C <- R'CR =(R'C)R=M*R;  where M=R'C
       %-----------------------------------------------------  
       % Covariance matrix of rotated data:
       M=C; M(compno,:)=R'*C(compno,:);    % M=R'*C;  M(dim,dim)
       Cnew=M; Cnew(:,compno)=M(:,compno)*R;       % C <- M*R
       
       % Find out which vector is the first principal component vector
       Cred=Cnew(compno,compno);
       C11=Cred(1,1); C22=Cred(2,2); C12=Cred(1,2); 
       if (C11<C22)
           idx=[2 1];
       else
           idx = [1 2];
       end
       
       % New correlation coefficients:
       dnew=diag(Cnew);   % dnew(dim,1)
       temp = sqrt(dnew(compno)*dnew'); %temp(2,dim)
       tmpnew=Cnew(compno,:)./temp;
       ccnew=cc; ccnew(compno,:)=tmpnew; %two new rows
       ccnew(:,compno)=tmpnew'; %two new columns       
   end